#!/usr/bin/env python

'''
Filter the output of SNV calling by VarScan (non somatic mode).
Assign remaining mutations to the genes of interest.
Franziska Singer, August 2015
'''

import sys
import os

if "-h" in sys.argv[1]:
	print "Filter the output of SNV calling by VarScan (non somatic mode)."
	print "Usage: python filterRawCalls_assignGenes_complete.py [vcfFileDirec] [outfileDirec] [checkStrands y/n] [filterHomoPolymer y/n] [filterSilent y/n] [freqFilter]"
	sys.exit(1)

'''
functions
'''	
def checkBases(allAllels):   # helper function that counts the number of occurrences of each base in reference and alternative allel 
	baseArr = [0,0,0,0]  # number of A, C, T, G that is found in the alleles
	for thisAllel in allAllels:
		if ("A" in thisAllel) or ("a" in thisAllel):
			baseArr[0] += 1
		if ("C" in thisAllel) or ("c" in thisAllel):
			baseArr[1] += 1
		if ("T" in thisAllel) or ("t" in thisAllel):
			baseArr[2] += 1
		if ("G" in thisAllel) or ("g" in thisAllel):
			baseArr[3] += 1
	return baseArr

'''
main
'''	

vcfFileDirec = sys.argv[1]	# read in the parameter for the input directory
outDirec = sys.argv[2]
checkStrands = sys.argv[3]				  # if set to y, perform an additional strand bias filtering
filterHomopolymer = sys.argv[4]   		  # if set to y, we check whether ref and alt allel show an indel which only consists of equal bases, e.g. AA -> A ; for ion torrent, this is likely to be a sequencing error
filterSilent = sys.argv[5]				  # if set to y, all mutations annotated with "synonymous" or "silent" are filtered
filterFrequency = float(sys.argv[6])

if not os.path.exists(outDirec):		# if the output directory does not exist, create it
    os.makedirs(outDirec)
	
# first read in target genes
dictGenesToSamples = {}
genes = ["MPL","NRAS","NOTCH2","IRF6","ALK","CASP8","IDH1","ERBB4","VHL","MLH1","CTNNB1","PIK3CA","TP63"]
dictGenes = {}   # dictionary that indexes the genes
for i in range(0,len(genes)):
	dictGenes[genes[i]] = i
	
sumOfPercents = 0.0  # used for some statistics

sampleDict = {}   # dictionary that maps sample names to their mutated genes
sampleNames = []
allFiles = 0

for file in os.listdir(vcfFileDirec):
	vcfFile = "%s%s" %(vcfFileDirec,os.path.basename(file))
	if os.path.isfile(vcfFile) and vcfFile.endswith(".vcf"):  # check if the file exists and if it is a vcf file
		name = os.path.basename(file).split(".vcf")[0]
		sampleDict[name] = [0] * len(genes)   # create new sample entry in dictionary, for each gene we initialize "zero" mutation counts
		sampleNames.append(name)
		allFiles += 1
		infile = open(vcfFile,'r')
		outName = "%s%s_filtered.vcf" %(outDirec,os.path.basename(file).split(".vcf")[0])
		outfile = open(outName,'w')  # create the filtered file

		allVariants = 0
		filteredVariants = 0

		for line in infile:
			if line.startswith("#"):   # these lines only need to be written to the filtered file
				outfile.write(line)
				continue
				
			allVariants += 1
			lineSplit = line.strip().split("\t")
			if ("WT=1" in line) or ("indelError" in line) or ("str10" in line):   # these are filter criteria applied by VarScan2. We use them as defaults.
				filteredVariants += 1
				continue
			
			if "y" in checkStrands:
				# check if one strand shows zero support, as an extension to the strand bias check by VarScan2
				strandFo = int(lineSplit[9].split(":")[12])	# get the number of variant supporting reads, first forward, then reverse strand
				strandRe = int(lineSplit[9].split(":")[13])
				if (strandFo == 0) or (strandRe == 0): # if either strand has only zero support, remove
					filteredVariants += 1
					continue
			
			if "y" in filterHomopolymer:
				refAllel = lineSplit[3]  # first get reference and alternative allel
				altAllel = lineSplit[4]
				
				baseArr = checkBases([refAllel,altAllel])  # call the function defined above
				sumDiffBases = 0
				for baseTemp in baseArr:   # go through the array returned by the function and check whether only one base appears in reference and alternative
					if baseTemp > 0:
						sumDiffBases += 1
				if sumDiffBases == 1:   # only one entry in array was different from zero -> only one base present
					filteredVariants += 1
					continue
			
			if "y" in filterSilent:
				# check if annotated as silent or synonymous
				if ("synonymous" in line) or ("silent" in line):
					filteredVariants += 1
					continue
			
			thisFreq = float(lineSplit[9].split(":")[6].split("%")[0])   # extract frequency, and in next line check if greater than threshold 
			if thisFreq <= filterFrequency:
				filteredVariants += 1
				continue
						
			annoTag = line.strip().split("\t")[7].split("ANN=")[1]		# this is necessary for the gene assignment
			geneName = annoTag.split("|")[3]							# get the gene name and count the new variant for this gene for this sample
			sampleDict[name][dictGenes[geneName]] += 1
			outfile.write(line)
			
		infile.close()
		outfile.close()
		sumOfPercents += float((float(filteredVariants)/float(max(1,allVariants)))*100.0)  # necessary to calculate average over all samples; max() to avoid crash if no variants present

		print "%s: Filtered %s of %s variants (%.3f%%)." %(os.path.basename(file),filteredVariants,allVariants,(float(filteredVariants)/float(max(1,allVariants)))*100.0)

print "\nOn average filtered %.3f%% of the variants." %(float(sumOfPercents)/float(allFiles))

# now create an overview table comparing identified mutations for the different groups
sampleNamesString = '\t'.join(sampleNames)
outfileOverview = open(outDirec+"overviewGeneMutations.txt",'w')
outfileOverview.write("Gene\tMutatedSamplesAll\tMutatedSamples_Resp\tMutatedSamples_Nonresp\t%s\n" %(sampleNamesString))  # table header

for i in range(0,len(genes)):
	mutatedSamples_total = 0				# initialize the per gene counts
	mutatedSamples_responder = 0
	mutatedSamples_nonresponder = 0
	sampleMutations = ""
	for thisName in sampleNames:	
		if sampleDict[thisName][i] > 0:			# check if the gene is mutated in the current sample
			mutatedSamples_total += 1
			if "Nonresp" in thisName:			# assign to the respective sample group
				mutatedSamples_nonresponder += 1
			else:
				mutatedSamples_responder += 1
		
		sampleMutations += "\t%s" %(sampleDict[thisName][i])
	outfileOverview.write("%s\t%s\t%s\t%s%s\n" %(genes[i],mutatedSamples_total,mutatedSamples_responder,mutatedSamples_nonresponder,sampleMutations))
outfileOverview.close()